/**
 * Temporal Quantified fuzzy DL's
 */

package monitorDb;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.*;
import java.util.*;


/**
 * The class for accessing a DBMS
 * 
 * @version 1.0 10/12/07
 * @author marco furlan
 */
public class Database {
    protected String url;
    protected String driver;
    protected String user;
    protected Properties prop;
    protected Connection con;
    protected Statement stm;
    protected ResultSet rs;
    protected ResultSetMetaData rsmt;
   
    
    /**
     * Generates a Database instance and connects it to a DBMS via jdbc. The String propFile
     * is  property file name with connection driver name and db simbolic name.
     * @param propFile the property file url
     * @throws Exception
     */
    public Database (String propFile) throws Exception {
    	if (propFile==null)
    		propFile = "monitorDb.properties";
        connect(propFile);       
    }
    /**
     * Generates a Database instance and connects it to a DBMS via jdbc, using a default 
     * property file with connection driver name and db simbolic name.
     * @throws Exception
     */
    public Database () throws Exception {
    	String
    	propFile = "monitorMyDb.properties";
        connect(propFile);       
    }
    /**
     * Connects to the DBMS read in the property file having p name
     * @param p the property file name
     * @throws FileNotFoundException
     * @throws IOException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    public void connect(String p) throws FileNotFoundException, IOException, 
	InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException{
		prop = new Properties ();
		prop.load(new FileInputStream(p));
		driver = prop.getProperty("jdbc.driver");
		url = prop.getProperty("jdbc.url");
		user = prop.getProperty("jdbc.username","");
		Class.forName(driver).newInstance();//non pi necessario con odbc, ancora necessario con mysql

	}
    /**
     * Executes a Select sql query to the database and return a table in a Vector<String> format.
     * First Vector element contains column names 
     * @param query the Select sql query
     * @return the table in a Vector<String> format
     * @throws SQLException
     */
    public synchronized Vector<String> getTable (String query) throws SQLException {
        con = DriverManager.getConnection (url, "root", "");
        stm = con.createStatement();
        Vector<String> v = new Vector<String>();        
        rs = stm.executeQuery (query);
        rsmt = rs.getMetaData();
        int l = rsmt.getColumnCount();
        String s = new String();
        for (int k = 1; k <= l; k++) {
            s += "  "+rsmt.getColumnName(k);                
        }
        v.addElement(s);
        while (rs.next()) {
            s="";
            for (int k = 1; k <= l; k++) {
                s += "  "+rs.getString(k);                
            }
            v.addElement(s);
        }
        rs.close();
        stm.close();
        con.close();
        return v;
    }
    
    /**
     * Executes an Update or Delete or Insert sql query for a table in the DB
     * @param query the UPDATE, DELETE or INSERT sql query  
     * @return <code>true</code> if update successfull, <code>false</code> otherwise
     * @throws SQLException
     */
    public synchronized boolean updTable (String query)
    throws SQLException {
        con = DriverManager.getConnection (url, "", "");
        stm = con.createStatement();
        boolean b = (stm.executeUpdate(query)>=1);
        stm.close();  
        con.commit();
        con.close();
        return b;
    }
       
    /**
     * Empty table test for a table in Vector<String> format
     * as obtained by getTabella() method
     * @param v table in Vector<String> format
     * @return <code>true</code> if table is empty, <code>false</code> otherwise
     */
    public synchronized boolean isEmptyTable (Vector<String> v) {
        int l = v.size();
        return (l==1);
    }
    
    /**
     * Deletes a tuple with key name <code>chiave</code> and value <code>value</code> 
     * from table with table name <code>tabella</code>
     * @param tabella the table name
     * @param chiave the key name
     * @param value the key value
     * @return <code>true</code> if deletion successful, <code>false</code> otherwise
     * @throws SQLException
     */
    public synchronized boolean delTuple (String tabella, String chiave, String value )
    throws SQLException {
        String query = "delete * from " + tabella + " where " + 
        chiave + " = '" + value + "'";
        return updTable(query);
    }
//    /**
//     * @param ar
//     * @throws Exception
//     */
//    public static void main (String ar[]) throws Exception {
//        Database db = new Database ();       
//        String query = "select * from CityTemperature";
//        Vector v = db.getTabella(query);
//        int l = v.size();
//        for (int k = 0; k < l; k++) {
//            System.out.println (v.elementAt(k));
//        }
//    }
}