/**
 * Temporal Quantified fuzzy DL's
 */
package prayPredator;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.GregorianCalendar;
/**
 * A JPanel class for pray-predator system graphic simulation, with current state 
 * in xml format export capability  
 * 
 * @version 1.0 10/12/07
 * @author marco furlan
 *
 */
public class XmlGraphPanel extends GraphPanel {

	/**
	 * Constructor in a PrayPredator object
	 * @param graph the PrayPredator object
	 */
	public XmlGraphPanel(PrayPredator graph) {
		super(graph);		
	}
	/* (non-Javadoc)
	 * @see prayPredator.GraphPanel#run()
	 */
	public void run() {
		Thread me = Thread.currentThread();
		while (relaxer == me) {
			relax(); 
			try {
				xmlExport();
			} catch (FileNotFoundException e1) {			
				e1.printStackTrace();
			} catch (JAXBException e1) {			
				e1.printStackTrace();
			} catch (DatatypeConfigurationException e) {			
				e.printStackTrace();
			} catch (IOException e) {			
				e.printStackTrace();
			}
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				break;
			}
		}
	}
	/**
	 * Exports the current state in xml format. Current state contains current time and current position for
	 * every Node.
	 * @throws JAXBException
	 * @throws DatatypeConfigurationException
	 * @throws IOException
	 */
	public synchronized void xmlExport() throws JAXBException, DatatypeConfigurationException, IOException{
		ObjectFactory factory = new ObjectFactory();
		CurrentStateType state = factory.createCurrentStateType();
		state.setTime(DatatypeFactory.newInstance().newXMLGregorianCalendar(new GregorianCalendar()));
		for (int i = 0; i < nnodes; i++) 	{
			Node n = nodes[i];
			NodeType node = factory.createNodeType();
			node.setCoordX(n.getX());
			node.setCoordY(n.getY());
			node.setName(n.getLbl());
			state.getNodes().add(node);
		}

		JAXBElement<CurrentStateType> cstate = (JAXBElement<CurrentStateType>) factory.createCurrentState(state);

		JAXBContext ctx = JAXBContext.newInstance("prayPredator");
		Marshaller m = ctx.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
		FileOutputStream out = new FileOutputStream("CurrentState.xml");
		m.marshal(cstate, out);
		out.flush();
		out.close();
	}
}
