/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import java.text.NumberFormat;
import java.util.HashMap;
//import java.util.HashSet;
import java.util.TreeSet;

/**
 * This class represents a Query in the generic form (Qt)(Q)R.C, where Qt is a 
 * temporal Quantifier, Q is a Quantifier, R is a Role, C is a Concept
 * @author marco furlan
 *
 */
public class RoleQuery implements Query {
	/**
	 * The Quantifier
	 */
	private Quantifier q;
	/**
	 * The temporal Quantifier
	 */
	private Quantifier tempq;
	/**
	 * The Role
	 */
	private Role r;
	/**
	 * The KnowledgeBase
	 */
	private KnowledgeBase kb;
	/**
	 * The ConcreteConcept
	 */
	private ConcreteConcept tc;
	/**
	 * The Concept
	 */
	private Concept c;
	
	private String str;
	private NumberFormat formatter;
	private NumberFormat formatter0; 

	/**
	 * Constructor for this RoleQuery, with null temporal quantifier
	 * @param q the Quantifier
	 * @param r the Role
	 * @param kb the KnowledgeBase
	 * @param tc the ConcreteConcept 
	 */
	public RoleQuery(Quantifier q, Role r, ConcreteConcept tc,KnowledgeBase kb) {
		super();
		this.setTempq(null);
		this.setQ(q);
		this.setR(r);
		this.setKb(kb);
		this.setTc(tc);
		this.setC(trap2Concept(tc));
		this.getKb().setQuery(this);
		setFormatter(3);
		setFormatter0(0);		
	}
	/**
	 * Constructor for this RoleQuery
	 * @param tempq the temporal Quantifier
	 * @param q the Quantifier
	 * @param r the Role
	 * @param tc the ConcreteConcept 
	 * @param kb the KnowledgeBase	 
	 */
	public RoleQuery(Quantifier tempq, Quantifier q, Role r, ConcreteConcept tc,KnowledgeBase kb) {
		super();
		this.setTempq(tempq);
		this.setQ(q);
		this.setR(r);
		this.setKb(kb);
		this.setTc(tc);
		this.setC(trap2Concept(tc));
		this.getKb().setQuery(this);
		setFormatter(3);
		setFormatter0(0);
	}
	/* (non-Javadoc)
	 * @see reasoner.Query#evaluate()
	 */
	public double evaluate() {
		double v=0.0;
		if (null==this.getTempq()){		//(Q)R.C
			str = "";	
			str += this.toString() +": ";
			if (q.isRelative()){				
				System.out.println("quantifier isRelative: true; ");
				v = q.eval(r, c);					
			}else
			{
				System.out.println("quantifier isRelative: false; ");
				v = 0.0;
//				todo
			}
			str += formatter.format(v) +"\n";
			return v;
		}else															//(Qt)R.C
			if(this.getTempq().isRelative()&& null==this.getQ()){

				str = this.getMeaning() + ": {";
				Role event = new Role("event", this.getKb());
				TreeSet<Double> tset = (TreeSet<Double>) this.getKb().getTimeSet();
				HashMap<String, ULineEvent> uls = this.getKb().getULineEvents();
				TreeSet<String> ulnames = new TreeSet<String>(uls.keySet());
				ULineEvent ul;
				Individual indd;
				double val;
				for (String ulname:ulnames){
					ul = (ULineEvent) uls.get(ulname);	
					event.getR().clear();
//					event.add(ul.getMyULine(), 1.0);
					for (double time : tset){
						if (time!=this.getKb().getSELF()){
							indd = ul.getIndividual(time);
							event.add(indd, 1.0);
						}
					}
					val= this.getTempq().eval(event.concat(this.getR()), c);
					str += "<" + ul.getName() + " : " + formatter.format(val) +">, ";
				}
				if (str.contains(","))
					str = str.substring(0, str.lastIndexOf(","));
				str += "}\n";
				return v;
			}
			else																//(Qt)(Q)R.C
				if(this.getTempq().isRelative()&&this.getQ().isRelative()){
					str = "";
					str += "(" + getQ().getName() + ")" + 
					getR().getName() + "." + getTc().getName()+": {";
					ULineEvent ull = new ULineEvent("ull",this.getKb());
					Concept ct= new Concept("c_t", this.getKb());
					Role rt = new Role("r_t", this.getKb());
					Concept cc= new Concept("cc", this.getKb());
					Role rr = new Role("rr", this.getKb());
					TreeSet<Double> tset = (TreeSet<Double>) this.getKb().getTimeSet();
					HashMap<String, ULineEvent> uls = this.getKb().getULineEvents();
					TreeSet<String> ulnames = new TreeSet<String>(uls.keySet());
					ULineEvent ul;
					Individual indd;
					double val, val1;
					for (String ulname:ulnames){
						if(!ulname.equals("ull")){
							ul = (ULineEvent) uls.get(ulname);				
							rt.add(ul.getMyULine(), 1.0);
						}
					}
					for (double time : tset){
						if (time!=this.getKb().getSELF()){
							ct.getEvents().clear();
							for (String ulname:ulnames){
								if(!ulname.equals("ull")){
									ul = (ULineEvent) uls.get(ulname);
									val = ul.get(time, c);
									ct.add(ul.getMyULine(), val);
								}
							}
							val1= this.getQ().eval(rt, ct);
							str += formatter0.format(time) + ": " + formatter.format(val1) +"; ";
							indd = new EventInd(ull.getName()+time,ull,time,getKb());
							cc.add(indd, val1);
							rr.add(indd, 1.0);
						}
					}
					str += "}\n";
					v = this.getTempq().eval(rr, cc);
					str += this.toString() + "          : " + formatter.format(v) + "\n";
					return v;
				}else  {
					// absolut quantifiers: da implementare
					return 0.0;
				}

	}
	
	/**
	 * Gets this RoleQuery evaluation in String format
	 * @return the String
	 */
	public String evaluateString(){
		this.evaluate();
		return str;
	}

	/**
	 * Converts a ConcreteConcept in a correspondent Concept
	 * @param t the ConcreteConcept
	 * @return the Concept
	 */
	private Concept trap2Concept(ConcreteConcept t){
		return t.toConcept();
	}
	/**
	 * @return the tc
	 */
	public ConcreteConcept getTc() {
		return tc;
	}

	/**
	 * @param tc the tc to set
	 */
	public void setTc(ConcreteConcept tc) {
		this.tc = tc;
	}

	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}

	/**
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
	}

	/**
	 * @return the q
	 */
	public Quantifier getQ() {
		return q;
	}

	/**
	 * @param q the q to set
	 */
	public void setQ(Quantifier q) {
		this.q = q;
	}

	/**
	 * @return the r
	 */
	public Role getR() {
		return r;
	}

	/**
	 * @param r the r to set
	 */
	public void setR(Role r) {
		this.r = r;
	}
	
	/* (non-Javadoc)
	 * @see reasoner.Query#toString()
	 */
	public String toString(){
		String s = "Query:  " + getMeaning() + " \n";
		return s;
	}
	
	/* (non-Javadoc)
	 * @see reasoner.Query#getMeaning()
	 */
	public String getMeaning(){
		String s = "";
		if (this.getTempq()!=null)
			s += "(Temp_" + this.getTempq().getName() + ")";
		if(getQ()!=null)
			s += "(" + getQ().getName() + ")";
			if(getR()!= null && getTc()!=null)
			s += getR().getName() + "." + getTc().getName();
		return s;
	}

	/**
	 * @return the c
	 */
	public Concept getC() {
		return c;
	}

	/**
	 * @param c the c to set
	 */
	public void setC(Concept c) {
		this.c = c;
	}

	/**
	 * @return the tempq
	 */
	public Quantifier getTempq() {
		return tempq;
	}

	/**
	 * @param tempq the tempq to set
	 */
	public void setTempq(Quantifier tempq) {
		this.tempq = tempq;
	}
	private void setFormatter(int n){
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(n);
	}
	private void setFormatter0(int n){
		formatter0 = NumberFormat.getNumberInstance();
		formatter0.setMaximumFractionDigits(n);
	}

}
