/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import java.util.TreeMap;

/**
 * This class represents the Role "distance"
 * @author marco furlan
 */
public class Distance extends Role {


	/**
	 * Constructs a new empty Distance
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public Distance(String name, KnowledgeBase kb) {
		super(name, kb);		
	}

	/**
	 * Constructs a new Distance with a given Role map
	 * @param r the Role map
	 * @param kb the KnowledgeBase
	 */
	public Distance(TreeMap<Individual, Double> r, KnowledgeBase kb) {
		super(r, kb);	
		this.setName("distance");	
	}

	/**
	 * Constructs a new empty Distance
	 * @param kb the KnowledgeBase
	 */
	public Distance(KnowledgeBase kb) {	
		super("distance", kb);
	}


	/**
	 * Gets a double value as euclidean distance between two points, given
	 * cartesian coordinates
	 * @param x1 the x coordinate for first point
	 * @param x2 the x coordinate for second point
	 * @param y1 the y coordinate for first point
	 * @param y2 the y coordiante for second point
	 * @return the euclidean distance
	 */
	public double getValue(double x1,double x2,double y1,double y2){
		return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
	}
	/* (non-Javadoc)
	 * @see reasoner.Role#toString()
	 */
	public String toString() {
		return "Role: " + 
		getName()+"FromTarget(X)=((X.x-Target.x)^2+(X.y-Target.y)^2)^1/2 \n";
	}

}
