/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;

import util.Point;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

/**
 * This class represents a Query in the form (Q)AopB, where A and B are 
 * ConcreteConcept, op is binary operator between Concepts and Q is Quantifier.
 * @author marco furlan
 */
public class OpQuery implements Query {
	/**
	 * The first ConcreteConcept
	 */
	private ConcreteConcept tc1;
	/**
	 * The second ConcreteConcept
	 */
	private ConcreteConcept  tc2;
	/**
	 * The first Concept
	 */
	private Concept c1; 
	/**
	 * The second Concept
	 */
	private Concept c2;
	/**
	 * The Quantifier
	 */
	private Quantifier q;
	/**
	 * The operator
	 */
	private String operator;
	/**
	 * The KnowledgeBase
	 */
	private KnowledgeBase kb;
	/**
	 * The last value map, for incremental evaluation
	 */
	private Map<String, Double> lastvalues;
	
	private Role is;
	private Concept d;
	private NumberFormat formatter; 

	
	/**
	 * Constructor by two ConcreteConcepts, binary operator and  KnowledgeBase
	 * @param tc1 the first ConcreteConcept
	 * @param operator the operator
	 * @param tc2 the second ConcreteConcept
	 * @param kb the KnowledgeBase
	 */
	public OpQuery(ConcreteConcept tc1, String operator,ConcreteConcept tc2, KnowledgeBase kb) {
		super();
		setTc1(tc1);
		setTc2(tc2);
		setC1(trap2Concept(tc1));
		setC2(trap2Concept(tc2));
		setOperator(operator);
		setKb(kb);
		getKb().setQuery(this);
		setLastvalues(new TreeMap<String, Double>());
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(6);
	}

	/** 
	 * Constructor by Quantifier, two ConcreteConcepts, binary operator and  KnowledgeBase
	 * @param q the Quantifier
	 * @param tc1 the first ConcreteConcept
	 * @param operator the operator
	 * @param tc2 the second ConcreteConcept
	 * @param kb the KnowledgeBase
	 */
	public OpQuery(Quantifier q, ConcreteConcept tc1, String operator,ConcreteConcept tc2, KnowledgeBase kb) {
		this(tc1,operator,tc2,kb);
		setQ(q);
	}

	/**
	 * Converts a ConcreteConcept in a Concept
	 * @param tc the ConcreteConcept
	 * @return the Concept
	 */
	private Concept trap2Concept(ConcreteConcept tc) {		
		return tc.toConcept();
	}


	/* (non-Javadoc)
	 * @see reasoner.Query#evaluate()
	 */
	public double evaluate() {
		String s = "";		
		return 0;
	}
	
	/**
	 * Evaluates this OpQuery and returns a String result. 
	 * @param incremental if <code>true</code> then incremental evaluation
	 * @param fuzzy if <code>true</code> then fuzzy evaluation for subsumption
	 * @return the result
	 */
	public synchronized String evaluateString(boolean incremental, boolean fuzzy) {
		HashMap<String,ULineEvent> uLines;
		Set<String> ulNames;
		Iterator<String> it ;
		String s = "", s1="";
		ULineEvent ul, ul1;
		Individual xlast,xo ;
		is = new Role ("is", getKb());
		String n = "("+this.getC1().getName()+")"+this.getOperator()+"("+this.getC2().getName()+")";
		d = new Concept(n, this.getKb());
		double lastVal, newVal, inverseVal, qval;
		if(this.getOperator().trim().equalsIgnoreCase("until")){
			s += "      ("+ this.getC1().getName() +")Until("+ this.getC2().getName() + "): {";
			uLines = this.getKb().getULineEvents();
			ulNames = uLines.keySet();			
			it = ulNames.iterator();
			while(it.hasNext()){
				String name = (String)it.next();
				ul = (ULineEvent) uLines.get(name);
				xlast = ul.getLast();
				xo = ul.getMyULine();
				if (incremental){
				lastVal = (getLastvalues().get(name)!=null?getLastvalues().get(name):0.0);
				
				newVal =this.getC1().until(this.getC2(), xlast, lastVal);
				getLastvalues().remove(name);
				getLastvalues().put(name, newVal);
				}else {
					newVal =this.getC1().until(this.getC2(), xlast);
				}
				is.add(xo, 1.0);
				d.add(xo, newVal);		//sostituisce il valore eventuale precedente
				if (newVal > 0.0)
					s += " <"+name+":"	+formatter.format(newVal)+"> ";	
					
			}
			s += "}\n" + s1 ;
		}else if(this.getOperator().trim().equalsIgnoreCase("subsumedNext")||
				this.getOperator().trim().equalsIgnoreCase("SubsNext")){
			HashMap<String, ArrayList<Double>> map = new HashMap<String, ArrayList<Double>>();
			ArrayList<Double> valList;
			Concept nextC, inter, interfull;
			Point linInterpol;
			String c1name =this.getC1().getName();
			uLines = this.getKb().getULineEvents();
			ulNames = new HashSet<String>();
			ulNames.addAll((Set<String>)uLines.keySet());//							
			it = ulNames.iterator();
			while(it.hasNext()){
				String name = (String)it.next();
				valList = new ArrayList<Double>();
				ul = (ULineEvent) uLines.get(name);
				xlast = ul.getLast();	
				xo = ul.getMyULine();
				nextC= this.getC1().getNextConcept(xlast);
				ul1 = ul.getWithoutLast();
				inter = this.getC1().intersection(ul1);
				interfull = this.getC1().intersection(ul);
				linInterpol = interfull.interpole();
				newVal = inter.subsumed(nextC, fuzzy);
				is.add(xo, 1.0);
				d.add(xo, newVal);		//sostituisce il valore eventuale precedente
				valList.add(newVal);
				inverseVal = nextC.subsumed(inter, fuzzy);
				valList.add(inverseVal);
				getKb().getULineEvents().remove(ul1.getName());
				valList.add(linInterpol.getX());
				valList.add(linInterpol.getY());
				map.put(name, valList);
			}
			s= this.subsNext2String(c1name, map);
		}
//		
		if (null!=this.getQ()){
			if (this.getQ().isRelative()){
//				System.out.println("quantifier isRelative: true; ");
				qval = this.getQ().eval(is, d);
			}else
				qval = 0.0;		//da fare
			s += "\n      "+this.getMeaning()+": "+
			formatter.format(qval)+"\n";	
		}
		return s;
	}
	
	public String subsNext2String(String concept, HashMap<String, ArrayList<Double>> map){
		String nextStr = "", prevStr = "", intStr = "", incrStr = "", decrStr = "", constStr = "";
		nextStr += "      SubsNext("+ concept + "): {";
		prevStr += "      SubsPrev("+ concept + "): {";
		intStr  += "      linear interpolation: {";
		incrStr += "      Incr("+ concept + "): {";
		decrStr += "      Decr("+ concept + "): {";
		constStr +="      Const("+ concept + "): {";
		Set<String> names = map.keySet();
		ArrayList<Double> values;
		for (String n:names){
			values = map.get(n);  
			if (values.get(0) > 0.0)
				nextStr += " <"+n+":"	+formatter.format(values.get(0))+"> ";	
			if (values.get(1) > 0.0)
				prevStr += " <"+n+":"	+formatter.format(values.get(1))+"> ";	
			if (values.get(0)>values.get(1))
				incrStr += " <"+n+":"	+formatter.format(values.get(0))+"> ";
			if (values.get(0)<values.get(1))
				decrStr += " <"+n+":"	+formatter.format(values.get(1))+"> ";
			if (values.get(0).compareTo(values.get(1))==0)
				constStr += " <"+n+":"	+formatter.format(values.get(0))+"> ";
			intStr += " <"+n+": angCoeff="+formatter.format(values.get(2))+", q= "+formatter.format(values.get(3))+"> ";
		}
		prevStr += "}\n";
		incrStr += "}\n";
		decrStr += "}\n";
		constStr += "}\n";
		intStr += "}\n";
		nextStr += "}\n" + prevStr + incrStr + decrStr + constStr + intStr;
		return nextStr;
		
	}

	/**
	 * @return the operator
	 */
	public String getOperator() {
		return operator;
	}


	/**
	 * @param operator the operator to set
	 */
	public void setOperator(String operator) {
		this.operator = operator;
	}


	/**
	 * @return the tc1
	 */
	public ConcreteConcept getTc1() {
		return tc1;
	}


	/**
	 * @param tc1 the tc1 to set
	 */
	public void setTc1(ConcreteConcept tc1) {
		this.tc1 = tc1;
	}


	/**
	 * @return the tc2
	 */
	public ConcreteConcept getTc2() {
		return tc2;
	}


	/**
	 * @param tc2 the tc2 to set
	 */
	public void setTc2(ConcreteConcept tc2) {
		this.tc2 = tc2;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		String s = "Query:  " + getMeaning() + "\n";
		return s;
	}

	/* (non-Javadoc)
	 * @see reasoner.Query#getMeaning()
	 */
	public String getMeaning(){
		String s = "";
		if(getQ()!=null)
			s += "(" + getQ().getName() + ")";
		if(getOperator().trim().equalsIgnoreCase("subsumedNext")){
			s += "SubsNext("+ getTc1().getName() + ")";
		}else
		if(getTc1()!=null && getOperator()!=null && getTc2()!=null){
			s += "(" + getTc1().getName() + ")";	
			
			s += getOperator() + "(" + getTc2().getName() + ")";
		}
		return s;	

	}

	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}


	/**
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
	}


	/**
	 * @return the c1
	 */
	public Concept getC1() {
		return c1;
	}


	/**
	 * @param c1 the c1 to set
	 */
	public void setC1(Concept c1) {
		this.c1 = c1;
	}


	/**
	 * @return the c2
	 */
	public Concept getC2() {
		return c2;
	}


	/**
	 * @param c2 the c2 to set
	 */
	public void setC2(Concept c2) {
		this.c2 = c2;
	}


	/**
	 * @return the lastvalues
	 */
	public Map<String, Double>  getLastvalues() {
		return lastvalues;
	}
	

	/**
	 * @param lastvalues the lastvalues to set
	 */
	public void setLastvalues(Map<String, Double> lastvalues) {
		this.lastvalues = lastvalues;
	}


	/**
	 * @return the q
	 */
	public Quantifier getQ() {
		return q;
	}


	/**
	 * @param q the q to set
	 */
	public void setQ(Quantifier q) {
		this.q = q;
	}

	/**
	 * @return the d
	 */
	public Concept getD() {
		return d;
	}

	/**
	 * @param d the d to set
	 */
	public void setD(Concept d) {
		this.d = d;
	}

	/**
	 * @return the is
	 */
	public Role getIs() {
		return is;
	}

	/**
	 * @param r the r to set
	 */
	public void setIs(Role r) {
		this.is = r;
	}

}
