/**
 * Temporal Quantified fuzzy DL's
 */
package reasoner;


import java.text.NumberFormat;
import java.util.ListIterator;
import java.util.TreeSet;
import java.util.Vector;


/**
 * This class represents a trapezoidal quantifier, with non negative
 * values only
 * @author marco furlan
 */
public class Quantifier extends Trapezoid {
	/**
	 * the name
	 */
	private String name = "Quantifier";

	private static double v1;
	private static double v2;
	/**
	 * the max value
	 */
	protected double maxval = Double.MAX_VALUE; 
	/**
	 * the min value is set to 0.0
	 */
	protected double minval = 0.0; 
	/**
	 * the KnowledgeBase
	 */
	protected KnowledgeBase kb;
	/**
	 * the constant SELF
	 */
	private double SELF;
	/**
	 * the constant NOW
	 */
	private double NOW;
	/**
	 * set to true if this Quantifier is relative, to false otherwise
	 */
	private boolean relative = true;
	
	
	/**
	 * Constructor by vertices, max and KnowledgeBase
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 * @param kb the KnowledgeBase
	 */
	public Quantifier(Double a1,Double b1,Double c1,Double d1, Double max, KnowledgeBase kb) {		
		setVertices(a1, b1, c1, d1, max);
		setName("Quantifier");
		setKb(kb);
		getKb().addQuantifier(this);
	}

	/**
     * Constructor by name, vertices, max and KnowledgeBase
	 * @param name the name
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 * @param kb the KnowledgeBase
	 */
	public Quantifier(String name, Double a1,Double b1,Double c1,Double d1, Double max, KnowledgeBase kb) {		
		setVertices(a1, b1, c1, d1, max);
		setName(name);
		setKb(kb);
		getKb().addQuantifier(this);
	}
	
	/**
	 * Constructor by name and KnowledgeBase
	 * @param name the name
	 * @param kb the KnowledgeBase
	 */
	public Quantifier(String name, KnowledgeBase kb) {
		super();		
		setKb(kb);
		//some cases
		if (name.trim().equalsIgnoreCase("Most")){				
			setVertices(0.5,0.6,1.0,1.0,1.0);
			setName("Most");
		}else if (name.trim().equalsIgnoreCase("One")){			
			setVertices(0.0,0.0,1.0,1.0,1.0);
			setName("One");
			setMeaning("One");
		}else if (name.trim().equalsIgnoreCase("Linear")){				
			setVertices(0.0,1.0,1.0,1.0,1.0);
			setName("Linear");
			setMeaning("Linear");
		}else if (name.trim().equalsIgnoreCase("All")){		
			setVertices(1.0,1.0,1.0,1.0,1.0);
			setName("All");
			setMeaning(getName());
		}else if (name.trim().equalsIgnoreCase("AlmostAll")){		
			setVertices(0.95,1.0,1.0,1.0,1.0);
			setName("AlmostAll");
			setMeaning(getName());
		}else if (name.trim().equalsIgnoreCase("AroundHalf")){		
				setVertices(0.4,0.5,0.5,0.6,1.0);
				setName("AroundHalf");
		}else if (name.trim().equalsIgnoreCase("Around1/4")){		
					setVertices(0.15,0.25,0.25,0.35,1.0);
					setName("Around(1/4)");
		}else if (name.trim().equalsIgnoreCase("Around10")){		
			setVertices(9.0,10.0,10.0,11.0,1000.0);
			setName("Around(10)");
		}else if (getAroundValue(name.trim())){
			if(v1>1.0)
				setVertices(v1-1.0,v1,v1,v1+1.0,1000.0);	
			else
				setVertices(v1-0.1,v1,v1,v1+0.1,1.0);
//			setName("Around("+ v1 + ")");
			setName();
		}else if(getBetweenValue(name.trim())){
				if(v1>1.0)
					setVertices(v1-1.0,v1,v2,v2+1.0,1000.0);
				else
					setVertices(v1-0.1,v1,v2,v2+0.1,1.0);
				//setName("Between("+ v1 + ")and(" + v2 + ")");
				setName();
		}else setName("Quantifier");
		getKb().addQuantifier(this);
	}
//	private static final String patternAround="Around\\(([0-9].*)\\)";
//	private static final String patternBetween="Between\\(([0-9].*)\\)" +
//			"and\\(([0-9].*)\\)";
//	public static boolean getAroundValue(String name){
//		Pattern p = Pattern.compile(patternAround, Pattern.CASE_INSENSITIVE);
//		Matcher m=p.matcher(name);
//		if (m.find()){
//			v1=Double.parseDouble(m.group(1).replace(",", "."));		
//			return true;
//		}else return false;
//	}
//	public static boolean getBetweenValue(String name){
//		Pattern p = Pattern.compile(patternBetween, Pattern.CASE_INSENSITIVE);
//		Matcher m=p.matcher(name);
//		if (m.find()){
//			v1=Double.parseDouble(m.group(1).replace(",", "."));
//			v2=Double.parseDouble(m.group(2).replace(",", "."));
//			return true;
//		}else
//		return false;
//	}
	/* (non-Javadoc)
	 * @see reasoner.Trapezoid#setName(java.lang.String)
	 */
	public	void setName(String name1) {
		name = name1;		
	}
	/**
	 * Sets the name by the shape, for some cases
	 */
	protected void setName() {		
		formatter = NumberFormat.getNumberInstance();
		formatter.setMaximumFractionDigits(2);
		if (getb().compareTo(getc())==0 && geta().compareTo(getd())!=0){
			setName ("Around(" + formatter.format(getb()) + ")");
		}
		if (getb().compareTo(getc())!=0 && geta().compareTo(getd())!=0){			
			setName ("Between(" + formatter.format(getb()) + ")and(" + formatter.format(getc()) +")");			
		}
		if (geta()==0.0 && getb()==0.0){
			setName ("LessThan("+ formatter.format(getc())+")");			
			
		}
	}
	/* (non-Javadoc)
	 * @see reasoner.Trapezoid#getName()
	 */
	public String getName() {
		if (name.equals("Quantifier"))setName();
		return name;		
	}
	/**
	 * Sets vertices and max value. 
	 * @param a1 first vertice
	 * @param b1 second vertice
	 * @param c1 third vertice
	 * @param d1 forth vertice
	 * @param max the max value
	 */
	protected void setVertices(Double a1,Double b1,Double c1,Double d1, Double max){
		super.setVertices(a1, b1, c1, d1);
		setMinval(0.0);
		setMaxval(max);
		if (max > 1.0)
			setRelative(false);		
		fixVertices();
	}

	
//	/**
//	 * @param d
//	 * @return
//	 */
//	private Double getValue(Double d){
//		if (d.equals(geta())||d.equals(getd()))
//			return 0.0;
//		else return 1.0;
//		
//	}
//	private Vector getValues(){
//		Vector<Double> v = new Vector<Double>();
//		v.add(0.0);
//		v.add(1.0);
//		v.add(1.0);
//		v.add(0.0);
//		return v;
//		
//	}
	
	
	/* (non-Javadoc)
	 * @see reasoner.Trapezoid#print()
	 */
	public void print() {		
		System.out.println("Quantifier name: "+ getName());
		System.out.println("Quantifier vertices:");
			System.out.printf("%.2f --> 0.0;  ",geta());
			System.out.printf("%.2f --> 1.0;  ",getb());
			System.out.printf("%.2f --> 1.0;  ",getc());
			System.out.printf("%.2f --> 0.0;  ",getd());
			System.out.printf("minVal: %.2f ;  ",getMinval());
			System.out.printf("maxVal: %.2f ;  ",getMaxval());
		System.out.println("");
	}
	
	/* (non-Javadoc)
	 * @see reasoner.Trapezoid#toString()
	 */
	public String toString(){
		String s = "";
		s += "Quantifier: " + this.getName() + ": ";
		s += "   " + this.getMeaning() + ": ";
		s += "   0.0/"+formatter.format(geta())+ " + 1.0/"+formatter.format(getb())+
		" + 1.0/"+formatter.format(getc())+ " + 0.0/"+formatter.format(getd());
		s += "\n";
		return s;
	}
	
	/**
	 * Sum between this Quantifier and a second one. Possible for 
	 * absolute quantifiers only!
	 * @param t the second Quantifier
	 * @return the sum
	 */
	public Quantifier sum(Quantifier t){
		setName("Quantifier");
		//setVertices(geta()+t.geta(),getb()+t.getb(),getc()+t.getc(), getd()+t.getd());
		super.sum(t);
		fixVertices();
		return this;
	}

	/**
	 * Difference between this Quantifier and a second one. Possible for 
	 * absolute quantifiers only!
	 * @param t the second Quantifier
	 * @return the difference
	 */
	public Quantifier diff(Quantifier t){
		setName("Quantifier");		
		super.diff(t);
		fixVertices();
		return this;
	}
	
	/**
	 * Product between this Quantifier and a second one. 
	 * @param t the second Quantifier
	 * @return the product
	 */
	public Quantifier prod(Quantifier t){
		setName("Quantifier");		
		super.prod(t);
		if (!t.isRelative()||t.getMaxval()>1.0){
			setRelative(false);
			setMaxval(Double.MAX_VALUE);
		}		
		fixVertices();
		return this;
	}

	/**
	 * Ratio between this Quantifier and a second one. Possible for 
	 * divisor not containing zero only!
	 * @param t the second Quantifier
	 * @return the ratio	 
	 */
	public Quantifier ratio(Quantifier t){
		setName("Quantifier");		
		super.ratio(t);
		if (!t.isRelative()||t.getMaxval()>1.0){
			setRelative(true);			
		}
		fixVertices();
		return this;
	}
	
//	/* (non-Javadoc)
//	 * @see reasoner.Trapezoid#intersection(reasoner.FuzzySet)
//	 */
//	public Quantifier intersection(FuzzySet f){
//		//to do
//		return this;
//	};
//	
//	/* (non-Javadoc)
//	 * @see reasoner.Trapezoid#union(reasoner.FuzzySet)
//	 */
//	public Quantifier union(FuzzySet f){
//		//to do
//		return this;
//	};
	/**
	 * Evaluates expression  (Q)R.C = Q(R_intersection_C / R), with Q this relative 
	 * Quantifier, R a Role and C a Concept.
	 * @param r the Role
	 * @param c the Concept
	 * @return the result
	 */
	public double eval(Role r, Concept c){
		double sigma=0.0;
	if (this.isRelative()){
		TreeSet<Double> lambda; 
		Role rc = r.intersection(c);
		System.out.println("rc: ");
		rc.print();
		lambda = r.totalLambda(rc);
		System.out.println("lambda: ");
		print(lambda);
		Vector<Double> v = new Vector<Double>(lambda);	
		ListIterator<Double> iter=  v.listIterator(v.indexOf(v.lastElement())+1);
		Double alfa1, alfa2, ratio, qvalue, diff;	
		Integer card_rc_alfa, card_r_alfa;
		while(iter.hasPrevious()){
			 alfa1 =(Double) iter.previous();
			 card_rc_alfa= rc.card(alfa1);
			 System.out.println("card di rc (con alfa= "+ alfa1+"): "+card_rc_alfa);
			 card_r_alfa= r.card(alfa1);
			 System.out.println("card di r (con alfa= "+ alfa1+"): "+card_r_alfa);
			 if (card_rc_alfa.equals(0) && card_r_alfa.equals(0))	{				//
				 ratio = 1.0;														//we set 0/0=1
			 }
			 else{
				 ratio = card_rc_alfa.doubleValue()/card_r_alfa.doubleValue();
			 }
			 System.out.printf("rapporto (con alfa= %.2f): %.2f \n",alfa1,ratio);
			 qvalue=qvalue(ratio);
			 System.out.printf("qvalue : %.2f \n", qvalue);
			 if (iter.hasPrevious()){
				 alfa2=(Double) iter.previous();
				 iter.next();//per tornare all'alfa corrente
			 }else{
				 alfa2=0.0;
			 }
			 System.out.printf("alfa1 : %.2f \n", alfa1);
			 System.out.printf("alfa2 : %.2f \n", alfa2);
			 diff=alfa1-alfa2;
			 System.out.printf("diff : %.2f \n", diff);
			 sigma += diff*qvalue;
			 System.out.printf("sigma : %.2f \n", sigma);
		}
		}
		return sigma;
	}
//	/**
//	 * evaluation with relative quantifier, of a quantified temporal 
//	 * subsumption from two Concept a, b
//	 * @param a the Concept temporal subsumed
//	 * @param c the Concept temporal subsuming
//	 * @return (Q)A subs C = Q(A_intersection_C / A)
//	 */
//	public double tempSubsumption(Concept a, Concept c){
//		double sigma=0.0;
//		if (getMaxval() <= 1.0){	
//		TreeSet<Double> lambda; 
//		Concept ac;		
//		a.setFirstLastTempMax(c);
//		ac = a.tempIntersection(c);		
//		System.out.println("ac: ");
//		ac.print();
//		lambda = a.totalUlLambda(ac);
//		System.out.println("lambda: ");
//		print(lambda);
//		Vector v = new Vector<Double>(lambda);	
//		ListIterator it=  v.listIterator(v.indexOf(v.lastElement())+1);
//		Double alfa1, alfa2, ratio, qvalue, diff;	
//		Integer cardAcAlfa, cardAalfa;
//		while(it.hasPrevious()){
//			 alfa1 =(Double) it.previous();
////			 cardAcAlfa= ac.tempCard(alfa1);
//			 cardAcAlfa= ac.card(alfa1 ,"time");
//			 System.out.println("card di ac (con alfa= "+ alfa1+"): "+cardAcAlfa);
////			 cardAalfa= a.tempCard(alfa1);
//			 cardAalfa= a.card(alfa1 , "time");
//			 System.out.println("card di a (con alfa= "+ alfa1+"): "+cardAalfa);
//			 ratio = cardAcAlfa.doubleValue()/cardAalfa.doubleValue();
//			 System.out.printf("rapporto (con alfa= %.2f): %.2f \n",alfa1,ratio);
//			 qvalue=qvalue(ratio);
//			 System.out.printf("qvalue : %.2f \n", qvalue);
//			 if (it.hasPrevious()){
//				 alfa2=(Double) it.previous();
//				 it.next();//per tornare all'alfa corrente
//			 }else{
//				 alfa2=0.0;
//			 }
//			 System.out.printf("alfa1 : %.2f \n", alfa1);
//			 System.out.printf("alfa2 : %.2f \n", alfa2);
//			 diff=alfa1-alfa2;
//			 System.out.printf("diff : %.2f \n", diff);
//			 sigma += diff*qvalue;
//			 System.out.printf("sigma : %.2f \n", sigma);
//		}
//		}
//		return sigma;
//	}
	
	/**
	  * Prints Lambda, the alpha level set for membership values, as a TreeSet<Double>
	 * @param lambda2 the Lambda 
	 */
	public void print(TreeSet<Double> lambda2) {
		Vector<Double> v = new Vector<Double>(lambda2);	
		ListIterator<Double> iter=  v.listIterator(v.indexOf(v.lastElement())+1);
		Object alfa;	
		while(iter.hasPrevious()){
			 alfa =iter.previous();		
			System.out.println("alfa :"+ alfa);
		}
		
	}
	
//	absolute quantifier
//	/**
//	 * evaluation with absolute quantifier, a Role or a Concept
//	 * @param c
//	 * @return evaluation of expression (Q)C
//	 */
//	public double eval(Concept c) {
////		maxval=Double.MAX_VALUE;
//		TreeSet<Double> lambda=c.getLambda();
//		System.out.println("lambda: ");
//	print(lambda);
//		double sigma=0.0;
//		Vector v = new Vector<Double>(lambda);	
//		ListIterator iter=  v.listIterator(v.indexOf(v.lastElement())+1);
//		Double alfa1, alfa2, qvalue, diff;	
//		Integer card_c_alfa;
//		while(iter.hasPrevious()){
//			 alfa1 =(Double) iter.previous();
//			 card_c_alfa= c.card(alfa1);
//			 System.out.println("card di c (con alfa= "+ alfa1+"): "+card_c_alfa);
//			 qvalue=qvalue(card_c_alfa.doubleValue());
//			 System.out.printf("qvalue : %.2f \n", qvalue);
//			 if (iter.hasPrevious()){
//				 alfa2=(Double) iter.previous();
//				 iter.next();//per tornare all'alfa corrente
//			 }else{
//				 alfa2=0.0;
//			 }
//			 System.out.printf("alfa1 : %.2f \n", alfa1);
//			 System.out.printf("alfa2 : %.2f \n", alfa2);
//			 diff=alfa1-alfa2;
//			 System.out.printf("diff : %.2f \n", diff);
//			 sigma += diff*qvalue;
//			 System.out.printf("sigma : %.2f \n", sigma);
//		}
//		
//		return sigma;
//	}

	
	
////	valutazione di un quantificatore relativo o assoluto
//	public Double qvalue1(Double ratio2) {
//		Double key1, key2, v1, v2, qval;		
//		Vector k = this.getKeys();	
//		Vector val = this.getValues();
//		ListIterator iter=  k.listIterator();
//		ListIterator iter1=  val.listIterator();
//		qval=0.0;
////		System.out.printf("0_qval: %.2f \n", qval);
//		while(iter.hasNext()){
//			key1 =(Double) iter.next();
////			System.out.printf("key1: %.2f \n", key1);
//			v1=(Double)iter1.next();
////			System.out.printf("v1: %.2f \n", v1);
//			if (iter.hasNext()){
//				key2 =(Double) iter.next();
////				System.out.printf("key2: %.2f \n", key2);
//				v2= (Double)iter1.next();
////				System.out.printf("v2: %.2f \n", v2);
//				iter.previous();
//				iter1.previous();
//			}else {
//				key2=getMaxval();
////				System.out.printf("key2: %.2f \n", key2);
//				v2=v1;
////				System.out.printf("v2: %.2f \n", v2);
//			}
//			if (ratio2<key1) return qval;
//			if(ratio2>=key1 && ratio2<=key2){
//				if (key1.equals(key2)) {
//					qval=Math.max(v1,v2);
////					System.out.printf("1_qval: %.2f \n", qval);
//					return qval;
//				}else 
//					if(v2==v1){
//					qval=v1;
////					System.out.printf("2_qval: %.2f \n", qval);
//					return qval;
//				}else{
//					if(v2>v1){
//						qval=(ratio2-key1)*(v2-v1)/(key2-key1)+v1;
////						System.out.printf("3_qval: %.2f \n", qval);
//						return qval;
//					}else{
//						qval=(key2-ratio2)*(v1-v2)/(key2-key1)+v2;
////						System.out.printf("4_qval: %.2f \n", qval);
//						return qval;
//					}
//				}
//			}
//		}	
//		return qval;
//	}
	
////	valutazione di un quantificatore relativo o assoluto
//	public Double qvalue(Double ratio2) {
//		Double key1, key2, v1, v2, qval;		
//		Vector k = getKeys();	
//		Vector val = getValues();
//		ListIterator iter=  k.listIterator();
//		ListIterator iter1=  val.listIterator();		
//		qval=0.0;
////		System.out.printf("0_qval: %.2f \n", qval);
//		while(iter.hasNext()){
//			key1 =(Double) iter.next();
////			System.out.printf("key1: %.2f \n", key1);
//			v1=(Double)iter1.next();
////			System.out.printf("v1: %.2f \n", v1);
//			if (iter.hasNext()){
//				key2 =(Double) iter.next();
////				System.out.printf("key2: %.2f \n", key2);
//				v2= (Double)iter1.next();
////				System.out.printf("v2: %.2f \n", v2);
//				iter.previous();
//				iter1.previous();
//			}else {
//				key2=getMaxval();
////				System.out.printf("key2: %.2f \n", key2);
//				v2=v1;
////				System.out.printf("v2: %.2f \n", v2);
//			}
//			if (ratio2<key1) return qval;
//			if (ratio2.equals(key1)){
//				if (key1.equals(key2)){
//					qval=Math.max(v1,v2);
////					System.out.printf("1_qval: %.2f \n", qval);
//					return qval;
//				}else{
//					qval=v1;
////					System.out.printf("2_qval: %.2f \n", qval);
//					return qval;
//				}
//			}else if(ratio2>key1 && ratio2<key2)
//				if(v2==v1){
//					qval=v1;
////					System.out.printf("2_qval: %.2f \n", qval);
//					return qval;
//				}else if(v2>v1){
//						qval=(ratio2-key1)*(v2-v1)/(key2-key1) + v1;
////						System.out.printf("3_qval: %.2f \n", qval);
//						return qval;
//				}else{
//						qval=(key2-ratio2)*(v1-v2)/(key2-key1) + v2;
////						System.out.printf("4_qval: %.2f \n", qval);
//						return qval;
//				}					
//			}		
//		return qval;
//	}

//	/**
//	 * @param c
//	 * @return
//	 */
//	public double eval(FuzzySet c){
//		TreeSet<Double> lambda=c.getLambda();
//		System.out.println("lambda: ");
//		print(lambda);
//		double sigma=0.0;
//		Vector<Double> v = new Vector<Double>(lambda);	
//		ListIterator<Double> iter=  v.listIterator(v.indexOf(v.lastElement())+1);
//		Double alfa1, alfa2, qvalue, diff;	
//		Integer card_c_alfa;
//		while(iter.hasPrevious()){
//			 alfa1 =(Double) iter.previous();
//			 card_c_alfa= c.card(alfa1, "any");
//			 System.out.println("card di r_c (con alfa= "+ alfa1+"): "+card_c_alfa);
//			 qvalue=qvalue(card_c_alfa.doubleValue());
//			 System.out.printf("qvalue : %.2f \n", qvalue);
//			 if (iter.hasPrevious()){
//				 alfa2=(Double) iter.previous();
//				 iter.next();//per tornare all'alfa corrente
//			 }else{
//				 alfa2=0.0;
//			 }
//			 System.out.printf("alfa1 : %.2f \n", alfa1);
//			 System.out.printf("alfa2 : %.2f \n", alfa2);
//			 diff=alfa1-alfa2;
//			 System.out.printf("diff : %.2f \n", diff);
//			 sigma += diff*qvalue;
//			 System.out.printf("sigma : %.2f \n", sigma);
//		}
//		
//		return sigma;
//	}
	
//	/**
//	 * @param alfa1
//	 * @return
//	 */
//	public Integer card(Double alfa1) {
//		// TODO Auto-generated method stub
//		return null;
//	}
	
	/* (non-Javadoc)
	 * @see reasoner.Trapezoid#getLambda()
	 */
	public TreeSet<Double> getLambda() {
		// TODO Auto-generated method stub
		return null;
	}
	/**
	 * @return the relative
	 */
	public boolean isRelative() {
		return relative;
	}
	/**
	 * Sets the relative attribute. If true, then set minval to 0.0 and maxval to 1.0.
	 * @param relative the relative to set
	 */
	public void setRelative(boolean relative) {
		this.relative = relative;
		if(relative){
			setMinval(0.0);
			setMaxval(1.0);
		}
	}
	/**
	 * @return the kb
	 */
	public KnowledgeBase getKb() {
		return kb;
	}
	/**
	 * Sets the KnowledgeBase and constants NOW and SELF
	 * @param kb the kb to set
	 */
	public void setKb(KnowledgeBase kb) {
		this.kb = kb;
		setNOW(getKb().getNOW());
		setSELF(getKb().getSELF());
	}
	/**
	 * @return the maxval
	 */
	public double getMaxval() {
		return maxval;
	}
	/**
	 * @param maxval the maxval to set
	 */
	public void setMaxval(double maxval) {
		this.maxval = maxval;
	}
	/**
	 * @return the minval
	 */
	public double getMinval() {
		return minval;
	}
	/**
	 * @param minval the minval to set
	 */
	public void setMinval(double minval) {
		this.minval = minval;
	}
	/**
	 * @return the NOW
	 */
	public double getNOW() {
		return NOW;
	}
	/**
	 * @param now the NOW to set
	 */
	public void setNOW(double now) {
		NOW = now;
	}
	/**
	 * @return the SELF
	 */
	public double getSELF() {
		return SELF;
	}
	/**
	 * @param self the SELF to set
	 */
	public void setSELF(double self) {
		SELF = self;
	}
}

