/**
 * Temporal Quantified fuzzy DL's
 */
package prayPredator;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

import javax.swing.JApplet;
import javax.swing.JPanel;


/**
 * A JPanel class for pray-predator system graphic simulation 
 * @author marco furlan
 *
 */
class GraphPanel extends Panel
implements Runnable, MouseListener, MouseMotionListener {
//	PrayPredator graph;
	JApplet graph;
	int nnodes;
	Node nodes[] = new Node[20];

	int nedges;
	Edge edges[] = new Edge[40];

	Thread relaxer;
	boolean stress = true;

	int numMouseButtonsDown = 0;

//	/**
//	 * Constructor in a PrayPredator object
//	 * @param graph the PrayPredator object
//	 */
//	public GraphPanel(PrayPredator graph) {
//		this.graph = graph;
//		addMouseListener(this);
//	}
	/**
	 * Constructor in a PrayPredator object
	 * @param graph the PrayPredator object
	 */
	public GraphPanel(JApplet graph) {
		this.graph = graph;
		addMouseListener(this);
	}

	/**
	 * Searches for Node with a given label
	 * @param lbl the node label
	 * @return the node index
	 */
	public int findNode(String lbl) {
		for (int i = 0; i < nnodes; i++) {
			if (nodes[i].lbl.equals(lbl)) {
				return i;
			}
		}
		return addNode(lbl);
	}

	/**
	 * Adds a new Node with a given label
	 * @param lbl
	 * @return
	 */
	public int addNode(String lbl) {
		Node n = new Node();
		n.x = 10 + 380 * Math.random();
		n.y = 10 + 380 * Math.random();
		n.lbl = lbl;
		nodes[nnodes] = n;
		return nnodes++;
	}

	/**
	 * Adds new Edge between two Nodes, with a given length
	 * @param from the first Node
	 * @param to the second Node
	 * @param len the length
	 */
	public void addEdge(String from, String to, int len) {
		Edge e = new Edge();
		e.from = findNode(from);
		e.to = findNode(to);
		e.len = len;
		edges[nedges++] = e;
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		Thread me = Thread.currentThread();
		while (relaxer == me) {
			relax(); 
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				break;
			}
		}
	}

	/**
	 * Sets new position for all nodes
	 */
	public synchronized void relax() {
		for (int i = 0; i < nedges; i++) {
			Edge e = edges[i];
			double vx = nodes[e.to].x - nodes[e.from].x;
			double vy = nodes[e.to].y - nodes[e.from].y;
			double len = Math.sqrt(vx * vx + vy * vy);
			len = (len == 0) ? .0001 : len;
			double f = (edges[i].len - len) / (len * 3);
			double dx = f * vx;
			double dy = f * vy;

			nodes[e.to].dx += dx;
			nodes[e.to].dy += dy;
			nodes[e.from].dx += -dx;
			nodes[e.from].dy += -dy;
		}

		for (int i = 0; i < nnodes; i++) {
			Node n1 = nodes[i];
			double dx = 0;
			double dy = 0;

			for (int j = 0; j < nnodes; j++) {
				if (i == j) {
					continue;
				}
				Node n2 = nodes[j];
				double vx = n1.x - n2.x;
				double vy = n1.y - n2.y;
				double len = vx * vx + vy * vy;
				if (len == 0) {
					dx += Math.random();
					dy += Math.random();
				} else if (len < 100 * 100) {
					dx += vx / len;
					dy += vy / len;
				}
			}
			double dlen = dx * dx + dy * dy;
			if (dlen > 0) {
				dlen = Math.sqrt(dlen) / 2;
				n1.dx += dx / dlen;
				n1.dy += dy / dlen;
			}
		}

		Dimension d = getSize();
		for (int i = 0; i < nnodes; i++) {
			Node n = nodes[i];
			if (!n.isPray()) {
				n.x += Math.max( -5, Math.min(5, n.dx));
				n.y += Math.max( -5, Math.min(5, n.dy));
				if (n.x < 0) {
					n.x = 0;
				} else if (n.x > d.width) {
					n.x = d.width;
				}
				if (n.y < 0) {
					n.y = 0;
				} else if (n.y > d.height) {
					n.y = d.height;
				}
				n.dx /= 2;
				n.dy /= 2;

			}else{
				n.dx = n.dirx*(-5+12*Math.random());
				n.dy = n.diry*(-5+12*Math.random());
				n.x += n.dx;
				n.y += n.dy;
				if (n.x < 0) {
					n.dirx *= -1;
					n.x = 0;
				} else if (n.x > d.width) {
					n.dirx *= -1;
					n.x = d.width;
				}
				if (n.y < 0) {
					n.diry *= -1;
					n.y = 0;
				} else if (n.y > d.height) {
					n.diry *= -1;
					n.y = d.height;
				}

			}

		}
		repaint();
	}

	Node pick;
	boolean pickpray;
	Image offscreen;
	Dimension offscreensize;
	Graphics offgraphics;

	final Color prayColor = Color.red;
	final Color selectColor = Color.pink;
	final Color edgeColor = Color.black;
	final Color nodeColor = new Color(250, 220, 100);
	final Color stressColor = Color.darkGray;
	final Color arcColor1 = Color.green;
	final Color arcColor2 = Color.yellow;
	final Color arcColor3 = Color.red;
	final Color backColor = Color.white;
	final Color bordColor = new Color(51, 255, 51);

	/**
	 * Paints a Node, given the Graphics, the Node and a FontMetrics
	 * @param g the Graphics
	 * @param n the Node
	 * @param fm the FontMetrics
	 */
	public void paintNode(Graphics g, Node n, FontMetrics fm) {
		int x = (int) n.x;
		int y = (int) n.y;
		g.setColor((n == pick) ? selectColor :
			(n.isPray() ? prayColor : nodeColor));
		int w = fm.stringWidth(n.lbl) + 10;
		int h = fm.getHeight() + 4;
		g.fillRect(x - w / 2, y - h / 2, w, h);
		g.setColor(Color.black);
		g.drawRect(x - w / 2, y - h / 2, w - 1, h - 1);
		g.drawString(n.lbl, x - (w - 10) / 2, (y - (h - 4) / 2) + fm.getAscent());
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#update(java.awt.Graphics)
	 */
	public synchronized void update(Graphics g) {
		Dimension d = getSize();
		if ((offscreen == null) || (d.width != offscreensize.width) ||
				(d.height != offscreensize.height)) {
			offscreen = createImage(d.width, d.height);
			offscreensize = d;
			if (offgraphics != null) {
				offgraphics.dispose();
			}
			offgraphics = offscreen.getGraphics();
			offgraphics.setFont(getFont());
		}

		offgraphics.setColor(backColor);
		offgraphics.fillRect(0, 0, d.width, d.height);
		offgraphics.setColor(bordColor);
		offgraphics.drawRect(0, 0, d.width-1, d.height-1);
		for (int i = 0; i < nedges; i++) {
			Edge e = edges[i];
			int x1 = (int) nodes[e.from].x;
			int y1 = (int) nodes[e.from].y;
			int x2 = (int) nodes[e.to].x;
			int y2 = (int) nodes[e.to].y;
			int len = (int) Math.abs(Math.sqrt((x1 - x2) * (x1 - x2) +
					(y1 - y2) * (y1 - y2)) - e.len);
			offgraphics.setColor((len < 10) ? arcColor1 :
				(len < 20 ? arcColor2 : arcColor3));
			offgraphics.drawLine(x1, y1, x2, y2);
			if (stress) {
				String lbl = String.valueOf(len);
				offgraphics.setColor(stressColor);
				offgraphics.drawString(lbl, x1 + (x2 - x1) / 2,
						y1 + (y2 - y1) / 2);
				offgraphics.setColor(edgeColor);
			}
		}

		FontMetrics fm = offgraphics.getFontMetrics();
		for (int i = 0; i < nnodes; i++) {
			paintNode(offgraphics, nodes[i], fm);
		}
		g.drawImage(offscreen, 0, 0, null);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		numMouseButtonsDown++;
		addMouseMotionListener(this);
		double bestdist = Double.MAX_VALUE;

		int x = e.getX();
		int y = e.getY();
		for (int i = 0; i < nnodes; i++) {
			Node n = nodes[i];
			double dist = (n.x - x) * (n.x - x) + (n.y - y) * (n.y - y);
			if (dist < bestdist) {
				pick = n;
				bestdist = dist;
			}
		}
		pickpray = pick.isPray();
		pick.setPray(true);
		pick.x = x;
		pick.y = y;

		repaint();
		e.consume();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		numMouseButtonsDown--;
		removeMouseMotionListener(this);

		pick.setPray(pickpray);
		pick.x = e.getX();
		pick.y = e.getY();
		if (numMouseButtonsDown == 0) {
			pick = null;
		}

		repaint();
		e.consume();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
	}
	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
		pick.x = e.getX();
		pick.y = e.getY();
		repaint();
		e.consume();
	}
	/**
	 * 
	 */
	public void start() {
		relaxer = new Thread(this);
		relaxer.start();
	}

	/**
	 * 
	 */
	public void stop() {    	
		relaxer = null;
	}


}


