/**
 * Temporal Quantified fuzzy DL's
 */
package prayPredator;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JApplet;
import javax.swing.JPanel;



/**
 * A JApplet class for simulating a pray-predator system
 * @author marco furlan
 *
 */
public class PrayPredator extends JApplet implements ActionListener{
	boolean isStandalone = false;
	String edges;
	String center;
	GraphPanel panel;
	JPanel controlPanel;
	JButton scramble; 
	static Point loc;


	/* (non-Javadoc)
	 * @see java.applet.Applet#init()
	 */
	public void init() {
		setLayout(new BorderLayout());
		panel = new GraphPanel(this);
		panel.setVisible(true);
		add("Center", panel);
		controlPanel = new JPanel();
		controlPanel.setBackground(new Color(219,229,241));
		add("South", controlPanel);
		scramble = new JButton("Scramble");
		controlPanel.add(scramble); 
		scramble.addActionListener(this); 

		try {
			edges = this.getParameter("edges", "Pray-One/50,Pray-Two/50,Pray-Three/50");
//			edges = this.getParameter("edges", "Pray-One/50,Pray-Two/50");
//			edges = this.getParameter("edges", "Pray-One/50");

			for (StringTokenizer t = new StringTokenizer(edges, ","); t.hasMoreTokens(); ) {
				String str = t.nextToken();
				int i = str.indexOf('-');
				if (i > 0) {
					int len = 100;
					int j = str.indexOf('/');
					if (j > 0) {
						len = Integer.valueOf(str.substring(j + 1)).intValue();
						str = str.substring(0, j);
					}
					panel.addEdge(str.substring(0, i), str.substring(i + 1), len);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		Dimension d = this.getSize();
		
		try {
			center = this.getParameter("center", "Pray");			
			if (center != null){
				Node n = panel.nodes[panel.findNode(center)];
				n.x =   d.width /7;
				n.y =   d.height/7;
				n.setPray(true);
				n.dirx = +1;
				n.diry = +1;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
//		panel.setVisible(true);
	}

	
	/* (non-Javadoc)
	 * @see java.applet.Applet#start()
	 */
	public void start() {
		panel.start();
	}

	
	/* (non-Javadoc)
	 * @see java.applet.Applet#stop()
	 */
	public void stop() {
		panel.stop();
	}

	
	/* (non-Javadoc)
	 * @see java.applet.Applet#destroy()
	 */
	public void destroy() {
		remove(panel);
		remove(controlPanel);
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == scramble) {
			Dimension d = getSize();
			for (int i = 0 ; i < panel.nnodes ; i++) {
				Node n = panel.nodes[i];
				if (!n.isPray()) {
					n.x += 100 + (d.width-20)*(1-2*Math.random());
					n.y += 100 + (d.height-20)*(1-2*Math.random());
				}
			}
			return;
		}
	}

	//Get Applet information
	/* (non-Javadoc)
	 * @see java.applet.Applet#getAppletInfo()
	 */
	public String getAppletInfo() {
		return "Title: PrayPredator \nAuthor: Marco Furlan";
	}
	
	/**
	 * Gets a parameter value
	 * @param key the parameter name
	 * @param def the parameter default value
	 * @return the parameter value
	 */
	public String getParameter(String key, String def) {
		return isStandalone ? System.getProperty(key, def) :
			(getParameter(key) != null ? getParameter(key) : def);
	}
	
	//Get parameter info
	/* (non-Javadoc)
	 * @see java.applet.Applet#getParameterInfo()
	 */
	public String[][] getParameterInfo() {
		String[][] info = {
				{"edges", "delimited string", "A comma-delimited list of all the edges.  It takes the form of 'C-n{/x}[,C-n{/x}]' where C is the name of center node (see 'center' parameter) and n is a node attached to the center node. You may (optionally) specify a length x separated from the edge name by a forward slash."},
				{"center", "string", "The name of the center node."}
		};
		return info;
	}

	
	/**
	 * Main method
	 * @param args the parameter "edge" and "center"
	 */
	public static void main(String[] args) {
		PrayPredator applet = new PrayPredator();
		applet.isStandalone = true;        

		JFrame frame;
		frame = new JFrame();
		frame.setTitle("Pray Predator Emulator");

		frame.add(applet, BorderLayout.CENTER);
//		applet.setBackground(Color.WHITE);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setSize(d.width*4/5, d.height*4/5);
//		frame.setLocation((d.width - frame.getSize().width) / 2,
//				(d.height - frame.getSize().height) / 4);
		
		loc = new Point((d.width - frame.getSize().width) / 2,
				(d.height - frame.getSize().height) / 4); 
		frame.setLocation(loc);
		applet.init();
		applet.start();
		frame.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		frame.setVisible(true);
	}
}
